package schelling;

/*-------------------------------------------------------------------------------

 File:			Agent.java
 Version:		1 (draft 1)
 Package:		schelling
 Class:			Agent
 
 Author:		Alan Mills
 
 Description:	This file contains the class 'Agent' for the Schelling segregation model.
 				The model supports Chapter seven of the report
 				"Complexity Science - an introduction (and invitation) for actuaries"
				written in 2010 for the SOA by Alan Mills.

 Methods:    	In addition to a constructor, the file contains the following methods:
               	1. step()
               	2. instance variable "getter" methods

 Notes:        	1.	See "Chapter six:  Artificial societies" of "Complexiy Science -
                  	an introduction (and invitation) for actuaries" to learn more about the structure
                  	of this program and about agent-based modeling in general.
               	2. 	Even though this program is relatively elementary, it is useful
                  	to review, because it employs certain agent-based programming
                  	techniques that are used for more complicated Complexity
                  	Science models.

-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
 TO DO
 1.   	 
------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 IMPORTED FILES
-------------------------------------------------------------------------------*/
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.parameter.Parameters;
import repast.simphony.query.space.grid.MooreQuery;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.grid.Grid;
import repast.simphony.util.ContextUtils;

public class Agent
{
	// Declare class instance variables
	private String agentID;
	private int groupID,
				neighborCount,
				neighborsLikeMeCount,
				happinessState;
	private double neighborPreferencePercent;

	// The class constructor. This is called by the Simphony runtime environment.
	public Agent(String inputID)
	{
		// Declare variables used in the constructor
		Parameters parameters;		
		int blueAgentsCount,
			redAgentsCount;
		double blueAgentFraction;
		
		// Note that 'this' refers to the object (Agent) being called.
		this.agentID = inputID;

		/*-------------------------------------------------
		 Get parameters from the "Parameters" entry screen
		 in the Repast Simphony runtime environment. So that
		 they will be properly ordered on the "Parameters"
		 entry screen, the parameter IDs have numbers as
		 their first character.
		---------------------------------------------------*/				
		parameters = RunEnvironment.getInstance().getParameters();
		
		// Define the variables
		this.neighborPreferencePercent = (Double)parameters.getValue("5NeighborPreferencePercent");
		blueAgentsCount = (Integer)parameters.getValue("1NumberOfBlueAgents");
		redAgentsCount = (Integer)parameters.getValue("2NumberOfRedAgents");
		
		/*-------------------------------------------------
		 The agent is randomly assigned a group ID of 0 or 1,
		 depending on the number of red and blue agents
		 (The integer agent counts are multiplied by 1.0
		 to change them to doubles, so that the division
		 will produce a double.)
		 For information about Simphony's random functions,
		 see the Simphony documentation section "Random
		 Numbers".
		---------------------------------------------------*/		
		blueAgentFraction = 1.0*blueAgentsCount/(1.0*blueAgentsCount + 1.0*redAgentsCount);
		if (RandomHelper.nextDoubleFromTo(0.0, 1.0) < blueAgentFraction)
		{
			this.groupID = 1;
		}
		else this.groupID = 0;
	
	}// end class constructor

/*=============================================================================
 Method:     	1. step()
 Description:  	Relocates the agent if the agent is not happy with its
 				neighbors.
 				The method is preceded by a schedule "annotation" that
 				instructs the Simphony scheduler to run the method starting
 				at tick 0, and for every tick thereafter. For information about
 				Simphony scheduling, see the Simphony documentation section 
 				"Working with the scheduler".
-------------------------------------------------------------------------------*/
	@ScheduledMethod(start=0,interval=1)
	public void step()
	{
		// Declare variables
		int	environmentColumns,
			environmentRows,
			x,
			y;
		Context<Agent> context;
		Grid<Agent> environment;
		MooreQuery<Agent> MooreQuery;
		
		// Get the agent's Context
		context = (Context)ContextUtils.getContext(this);
		
		// Get the environment (grid) from the Context
		environment = (Grid)context.getProjection("2DGrid");
		
		/*-------------------------------------------------
		 The agent will look for a new site if it is
		 unhappy with its current location and will continue
		 to look for a new site until a satisfactory site
		 is found. Start with the agent's happiness state
		 set to 0 (unhappy).
		---------------------------------------------------*/				
		
		happinessState = 0;

		// Loop until the agent becomes happy
		while(happinessState == 0)
		{
			// Query the agent's Moore neighbors
			MooreQuery = new MooreQuery<Agent>(environment, this);

			// Initialize variables
			neighborsLikeMeCount = 0;
			neighborCount = 0;

			// Check Moore neighbors and count the number of like types
			for (Agent agent : MooreQuery.query())
			{
				neighborCount++;
				
				if (agent.getGroupID() == this.getGroupID())
					neighborsLikeMeCount++;
				
			}// end for

			/*-------------------------------------------------
			 If the number of neighbors like the agent is above
			 the agents preference percentage, set the agent to
			 happy. Otherwise, move to another spot.
			 Note that the grid cells are numbered starting
			 with 0 (not 1).
			---------------------------------------------------*/		
			if (neighborsLikeMeCount >= neighborPreferencePercent * neighborCount)
			{
				happinessState = 1;
			}
			else
			{
				environmentColumns  = environment.getDimensions().getWidth();
				environmentRows = environment.getDimensions().getHeight();

				x = RandomHelper.nextIntFromTo(0, environmentColumns-1);
				y = RandomHelper.nextIntFromTo(0, environmentRows-1);

				// If the location is not empty (!= null), find another location
				while(environment.getObjectAt(x,y) != null)
				{
					x = RandomHelper.nextIntFromTo(0, environmentColumns-1);
					y = RandomHelper.nextIntFromTo(0, environmentRows-1);
					
				}// end while

				// Move to new empty location
				environment.moveTo(this, x,y);
				
			}// end else
		}// end while

	
	}// end step()
/*=============================================================================*/

/*=============================================================================
 Method:     	2. Instance return methods ('getters')
 Description:  	Methods to return instance variables.
 ------------------------------------------------------------------------------*/
	public String getAgentID()
	{
		return this.agentID;
	}
	
	public String toString()
	{
		return this.agentID;
	}

	public int getGroupID()
	{
		return this.groupID;
	}

	public int getNeighborCount()
	{
		return this.neighborCount;
	}
	
	public int getNeighborsLikeMeCount()
	{
		return this.neighborsLikeMeCount;
	}
	
	public int getHappinessState()
	{
		return this.happinessState;
	}

	public double getNeighborPreferencePercent()
	{
		return this.neighborPreferencePercent;
	}
	
/*=============================================================================*/
	
}// end class Agent
