package schelling;

/*-------------------------------------------------------------------------------

File:			SchellingModel.java
Version:		1
Package:		schelling
Class:			SchellingModel

Author:			Alan Mills
				
Description:	This file contains the class 'SchellingModel' for the Schelling segregation model.
				The model supports Chapter seven of the report
				"Complexity Science - an introduction (and invitation) for actuaries"
				written in 2010 for the SOA by Alan Mills.

Methods:    	1. build()

Notes:        	1.	See "Chapter six:  Artificial societies" of "Complexiy Science -
                 	an introduction (and invitation) for actuaries" to learn more
                 	about this program and about agent-based modeling in general.
              	2. 	Even though this program is relatively elementary, it is useful
                 	to review, because it employs agent-based programming
                 	techniques that are used for more complicated Complexity
                 	Science models.

-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
TO DO
1.  
------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
IMPORTED FILES
-------------------------------------------------------------------------------*/
import repast.simphony.context.Context;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.RandomGridAdder;
import repast.simphony.space.grid.WrapAroundBorders;

// This is the first class called by the Simphony environment.
// There are no class instance variables and there is no explicit constructor,
// because they are not needed.
public class SchellingModel implements ContextBuilder<Object>
{
	
	/*=============================================================================
	 Method:     	1. build()
	 Description:  	Initializes the Schelling segregation model.
	 				It gets parameters from the Simphony runtime environment,
	 				populates the context with agents, and creates the 2D grid
	 				environment (a "projection") on which agents move.
	 				A 'build()' method is required for classes that implement
	 				the Simphony ContextBuilder class.
	-------------------------------------------------------------------------------*/
	public Context<Object> build(Context<Object> contextSchellingModel)
	{
		// Declare variables.
		Parameters parameters;
		int blueAgentsCount,
			redAgentsCount,
			environmentColumns,
			environmentRows;
		Agent agent; // Agent is a class defined in this package
		long agentCounter;
		
		/*-------------------------------------------------
		 Get parameters from the "Parameters" entry screen
		 in the Repast Simphony runtime environment. So that
		 they will be properly ordered on the "Parameters"
		 entry screen, the parameter IDs have numbers as
		 their first character.
		---------------------------------------------------*/		
		parameters = RunEnvironment.getInstance().getParameters();
		
		// Define the variables
		blueAgentsCount = (Integer)parameters.getValue("1NumberOfBlueAgents");
		redAgentsCount = (Integer)parameters.getValue("2NumberOfRedAgents");
		environmentRows = (Integer)parameters.getValue("3EnvironmentRows");
		environmentColumns = (Integer)parameters.getValue("4EnvironmentColumns");
		
	    // Create the agents and add them to the context.
		for(agentCounter = 0; agentCounter < (blueAgentsCount + redAgentsCount); agentCounter++)
		{
			agent = new Agent("Agent-" + agentCounter);
			contextSchellingModel.add(agent);
		}
		
		/*-------------------------------------------------
		 Create the 2D grid with GridFactory().
		 The grid is a Simphony "projection". Projections
		 define the relationships among agents.
		 Projections are created using "factory" classes.
		 The factory for a grid projection is the 
		 GridFactory().
		 Its inputs are:
	 		- the grid name (this comes from the model.score)
	 		- the context in which to place the grid (the method parameter)
	 		- border specification
	 		- random adder class for populating the grid randomly
	 		  with agents
	 		- boolean value for multiple occupancy
	 		  (false = single occupancy)
	 		- grid dimensions (columns, rows)
		---------------------------------------------------*/	 			
		GridFactoryFinder.createGridFactory(null).createGrid("2DGrid",
				contextSchellingModel, 
				new GridBuilderParameters<Object>(new WrapAroundBorders(), 
				new RandomGridAdder<Object>(),
				false,
				environmentColumns, environmentRows));
		
		return contextSchellingModel;
		
	}// end build()
	
}// end class SchellingModel
